@@ -47,6 +47,17 @@ function WebSocketConnection(socket, extensions, protocol, maskOutgoingPackets,
     // Superclass Constructor
     EventEmitter.call(this);
 
+    this._pingListenerCount = 0;
+    this.on('newListener', function(ev) {
+        if (ev === 'ping'){
+            this._pingListenerCount++;
+        }
+      }).on('removeListener', function(ev) {
+        if (ev === 'ping') {
+            this._pingListenerCount--;
+        }
+    });
+
     this.config = config;
     this.socket = socket;
     this.protocol = protocol;
@@ -606,10 +617,29 @@ WebSocketConnection.prototype.processFrame = function(frame) {
             break;
         case 0x09: // WebSocketFrame.PING
             this._debug('-- Ping Frame');
-            this.pong(frame.binaryPayload);
+
+            if (this._pingListenerCount > 0) {
+                // logic to emit the ping frame: this is only done when a listener is known to exist
+                // Expose a function allowing the user to override the default ping() behavior
+                var cancelled = false;
+                var cancel = function() { 
+                  cancelled = true; 
+                };
+                this.emit('ping', cancel, frame.binaryPayload);
+
+                // Only send a pong if the client did not indicate that he would like to cancel
+                if (!cancelled) {
+                    this.pong(frame.binaryPayload);
+                }
+            }
+            else {
+                this.pong(frame.binaryPayload);
+            }
+
             break;
         case 0x0A: // WebSocketFrame.PONG
             this._debug('-- Pong Frame');
+            this.emit('pong', frame.binaryPayload);
             break;
         case 0x08: // WebSocketFrame.CONNECTION_CLOSE
             this._debug('-- Close Frame');
